/** \file
	\brief armscope main source file
	\author Tomasz Ostrowski
*/

#include "board.h"
#include <stdlib.h>
//#include "dbgu.h"
#include "cdc_enumerate.h"
#include "board_armscope.h"
#include "delay.h"
#include "adc.h"
#include "timer.h"
#include "usb.h"
#include "buzzer.h"
#include "command.h"
#include "armscope_settings.h"

#if defined(__WinARM__) && !defined(UART_COM)
#warning "make sure syscalls.c is added to the source-file list (see makefile)"
#endif

int main ( void )
{
    char data[14336];
    char dataIn[21];
    char trig_sample=0, trig_sample2=0, trig_sample3=0, trig_sample4;
	struct S_ARMSCOPE_SETTINGS settings;
	int trig = 0;
    unsigned int length=0;
    unsigned char data_available = 0;

    ADCInit();
    //* - Set the Clock frequency which supplies the timer, the mode
    AT91F_TC_Open ( AT91C_BASE_TC0,
                    (AT91C_TC_ASWTRG_CLEAR | AT91C_TC_ACPC_CLEAR | AT91C_TC_ACPA_SET | AT91C_TC_LDRA_RISING |
                     AT91C_TC_WAVE  | AT91C_TC_WAVESEL_UP_AUTO | TC_MCK_DIV ),
                    TIMER_ID);

    ConfigureIO();

    beep();
    beep();

    // Enable User Reset and set its minimal assertion to 960 us
    AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);

    // Init USB device
    AT91F_USB_Open();

    //defaultowa inicjalizacja linii sterujacych
	//	clearX10();
    clearG2();
    clearG3();
    setG1();
    clearACDC();

    while (1)	
    {
        // Check enumeration
        if (pCDC.IsConfigured(&pCDC))
        {
            enum E_ACTION_REQUEST action = REQUEST_NONE;
			
			length += pCDC.Read(&pCDC, &dataIn[length], 3);		//non-blocking!
            if (length>=3)
            {                
                if (dataIn[2]==0x00)	//end of message detected - correct frame
                {
                    action = ExecuteCmd(dataIn, length, &settings);
                }
				length=0;
            }

            if (action == REQUEST_TRIGGER)
				trig = 1;
			else
			{
				if (settings.trigger_type != ARMSCOPE_TRIGGER_MANUAL && !data_available)
				{
					//test trigger condition
					trig_sample4 = trig_sample3;
					trig_sample3 = trig_sample2;
					trig_sample2 = trig_sample;
					while (!((AT91C_BASE_ADC->ADC_SR) & (1<<CHANNEL)));	//wait for sample
					trig_sample= (char)(AT91C_BASE_ADC->ADC_CDR3);
					
					//if (settings.trigger_type == ARMSCOPE_TRIGGER_SLOPE_FALLING ||
					//	settings.trigger_type == ARMSCOPE_TRIGGER_SLOPE_RAISING)
					//	beep();
					
					//detect slope
					if ( settings.trigger_type == ARMSCOPE_TRIGGER_SLOPE_FALLING &&
						(trig_sample>settings.trigger_level) && (trig_sample2>=settings.trigger_level) &&
						(trig_sample3<=settings.trigger_level) && (trig_sample4<settings.trigger_level) )	//falling (remember, negative gain)
					{
						trig = 1;
					} else if ( settings.trigger_type == ARMSCOPE_TRIGGER_SLOPE_RAISING &&
						(trig_sample<settings.trigger_level) && (trig_sample2<=settings.trigger_level) &&
						(trig_sample3>=settings.trigger_level) && (trig_sample4>settings.trigger_level) )	//opadanie napiecia (pamietac, ze wzm. jest odwracajacy)
					{
						trig = 1;
					}
				}
			}

            if (trig==1)
            {
                trig = 0;
                for (int n=0; n < settings.capture_buf_size; n++)
                {
                    while (!((AT91C_BASE_ADC->ADC_SR) & (1<<CHANNEL)));	//wait for sample
                    data[n]= (char)(AT91C_BASE_ADC->ADC_CDR3);
                }
                data_available = 1;	//data available to send to PC
            }

            if (action == REQUEST_DATA_FETCH)	//transfer request from PC
            {
                if (data_available)
                {
                    pCDC.Write(&pCDC, data, settings.capture_buf_size);	//wyslij duzo danych
                    data_available = 0;
                }
                else
                {
                    for (int n=0; n < 128;)			//fill special "no data" frame
                    {
                        data[n++] = (unsigned char)255;
                        data[n++] = (unsigned char)0;
                    }
                    pCDC.Write(&pCDC, data, 128);		//send "no data" frame
                }
            }
			
        }	//if (pCDC.IsConfigured(&pCDC))
    }	//while (1)
}



